;;; ------------------------------------------------------------
;;; PC SCHEME START-UP FILE -- COMPILE AND CONVERT TO PATCH.PCS
;;; ------------------------------------------------------------
;;; An example of a PC Scheme patch file for use with Menus for
;;; Scheme (tm)

;;; ------------------------------------------------------------
;;; Modifed Error Handler

;;; Suppresses the inspector when PC Scheme is not in debug
;;; mode.  Author's note: I have found this simple modification
;;; to the error handler reduces my irritation with the TI
;;; version so much that I want to share it with all PC Scheme
;;; programmers.  -- M. Goldberg
(define (*user-error-handler* code
                              message
                              irritant
                              sys-error-handler)
  (if pcs-debug-mode
    (sys-error-handler)
    (begin
      (writeln "ERROR: " message)
      (writeln "CAUSED BY: " irritant)
      (reset))))

;;; ------------------------------------------------------------
;;; Menu-Based User Interface Loader

;;; If the predicates mouse? is set to #f, MENKEY.FSL is loaded.
;;; Otherwise, MENMOU.FSL is loaded.  If the predicate xed? is
;;; NOT #f, MENED.FSL is loaded.
(let ((mouse? #f)
      (xed? #f))
  (fast-load (string-append pcs-sysdir "\\SCOOPS.FSL"))
  (fast-load (string-append pcs-sysdir "\\MENUS.FSL"))
  (if mouse?
    ;; Load the mouse-driven top-level for the menu-based user
    ;; interface.
    (fast-load (string-append pcs-sysdir "\\MENMOU.FSL"))
    ;; Load the keyboard-driven top-level for the menu-based
    ;; user interface.
    (fast-load (string-append pcs-sysdir "\\MENKEY.FSL")))
  (fast-load (string-append pcs-sysdir "\\MENLD.FSL"))
  (fast-load (string-append pcs-sysdir "\\MENCO.FSL"))
  (fast-load (string-append pcs-sysdir "\\MENDOS.FSL"))
  (if xed?
    ;; Load the external editor menu module.
    (fast-load (string-append pcs-sysdir "\\MENED.FSL")))
  (if mouse?
    ;; Set the console window attributes to agree with what is
    ;; specified by the mouse menu "Clear Screen" item.
    (clr-console)))

;;; ------------------------------------------------------------
